
#include "lpc1114_ump.h"

#include <stdio.h>


static const char * statename (const int _state)
{
    switch (_state)
    {
    case 0: return "Idle";
    case 1: return "Receive";
    case 2: return "ReceiveStuff";
    case 3: return "Transmit";
    case 4: return "WaitLineEmpty";
    }
    return "?";
}


static struct UmpInterface ump0;

static void serve_uart0_interrupt(const uint8_t IIRValue)
{
    // broken char -> collision
//    ump_handle_rxerror(&ump0);

    /* Receive Data Available or Character timeout indicator */
//    ump_handle_received_data(&ump0, c);

    /* transmit holding register empty */
//    ump_handle_transmitter_empty(&ump0);
}


static inline void InitializeUart(void)
{
}


void ump_lpc1114_stop_uart (struct UmpInterface *_ump)
{
    (void)_ump;
}

void ump_lpc1114_start_uart (struct UmpInterface *_ump)
{
    (void)_ump;
}

static int g_next_tx_octet = -1;
void ump_lpc1114_uart_send_octet(struct UmpInterface *_ump, const uint8_t _octet)
{
    (void)_ump;
    printf ("[%02X] state:%s\n", _octet, statename(_ump->state));
    g_next_tx_octet =  _octet;
}

static void _ump_timer_callback_handler()
{
    ump_handle_timer_interrupt(&ump0);
}

void ump_lpc1114_init_timer (struct UmpInterface *_ump)
{
    // init posix timer
}

static void ump_lpc1114_restart_timeout(struct UmpInterface * _ump, const uint16_t _timeout_ms)
{
    (void)_ump;
    // start posix timer
}

static void ump_lpc1114_stop_timeout(struct UmpInterface * _ump)
{
    (void)_ump;
    // stop_posix_timer
}



static void frameReceivedCallback(struct UmpInterface *_ump, struct UmpFrame *_frame)
{
    printf ("frame received: src=%02X dst=%02X flags=%02X, size=%02X :",
	    _frame->source, _frame->dest, _frame->flags, _frame->size);
    int i;
    for (i = 0; i < _frame->size; ++i)
    {
	printf (" %02X", _frame->data[i]);
    }
    printf ("\n");

    ump_send_frame(_ump, _frame);
}


//******************************************
struct UmpCallbacks ump0_callbacks =
{
    .start_uart = ump_lpc1114_stop_uart,
    .stop_uart = ump_lpc1114_start_uart,
    .uart_send_octet = ump_lpc1114_uart_send_octet,

    .init_timer = ump_lpc1114_init_timer,
    .restart_timeout = ump_lpc1114_restart_timeout,
    .stop_timeout = ump_lpc1114_stop_timeout,

    .frameReceivedCallback = frameReceivedCallback
};


static const struct UmpFrame testFrame =
{
    0x01, // source
    0x02, // dest
    0x03, // type
    0x04, // size
    {
	'A',
	0x55,
	0x07,
	27,
    }
};

/*
 * Application entry point.
 */
int main(void)
{
    ump_init(&ump0, &ump0_callbacks);
    InitializeUart();

    ump_start(&ump0);


#if 0 // test1
    const uint8_t rxdata[14] = {
	0x55,
	0xAA,
	0x01,
	0x02,
	0x03,
	0x04,
	0xF4,
	0x41,
	0x1B,
	0x05,
	0x07,
	0x1B,
	0x02,
	0xF2
    };

    int i;
    for (i = 0; i < sizeof(rxdata); ++i)
    {
	printf ("state:%s ", statename(ump0.state));
	printf ("rxstate:%d ", ump0.rx.rxstate);
	printf (" rx:%02X ", rxdata[i]);
	ump_handle_received_data(&ump0, rxdata[i]);
	printf ("state':%s ", statename(ump0.state));
	printf ("rxstate:%d ", ump0.rx.rxstate);
	printf ("\n");
    }


    _ump_timer_callback_handler();

    while (1)
    {
	if (g_next_tx_octet >= 0)
	{
	    const int t = g_next_tx_octet;
	    g_next_tx_octet = -1;
	    ump_handle_received_data(&ump0, t);
	}
    }
#endif


    while (1)
    {
	sleep(1);
	ump_send_frame(&ump0, &testFrame);
	_ump_timer_callback_handler();
/*
	while (1)
	{
	    if (g_next_tx_octet >= 0)
	    {
		const int t = g_next_tx_octet;
		g_next_tx_octet = -1;
		ump_handle_received_data(&ump0, t);
	    }
	}
*/
    }


    ump_stop(&ump0);

    return 0;
}

